-- ChatSubs 3.00
-- by Travis S. Casey (efindel@gmail.com)

-- Ideas for 3:
-- - add configuration panel, allow setting character's pronoun, pet's pronouns

-- detect whether ChatSubs has already been loaded
if GetAddOnMetadata("ChatSubs", "Version") then
  local version = tonumber(GetAddOnMetadata("ChatSubs", "Version"))
  if version then
    -- if version is nil, it's an older version using the x.y.z 
    -- version numbering, so we're a newer version
    local myversion = tonumber(GetAddOnMetadata("AddEmote", "Version"))
    if myversion and ( myversion <= version ) then
      -- we can use the loaded ChatSubs, so don't load this one
      -- "return" when you aren't in a function should just exit 
      -- the current file
      return
    end
  end
end
-- if we make it here, then either ChatSubs isn't there, it's an older
-- version, or this *is* ChatSubs, not AddEmote

-- set up a namespace for our non-local functions
ChatSubs = {}
ChatSubs.CurrentText = ""

-- string declarations for various functions to use
local Nominative = { "it", "he", "she" }
local Objective = { "it", "him", "her" }
local Possessive = { "its", "his", "her" }
local Possessive2 = { "its", "his", "hers" }
local Noun = { "person", "male", "female" }
local ChildNoun = { "child", "boy", "girl" }
local LadNoun = { "child", "lad", "lass" }
local AdultNoun = { "person", "man", "woman" }
local MasterNoun = { "boss", "master", "mistress" }
local LordNoun = { "noble", "lord", "lady" }
local SirNoun = { "ser", "sir", "madam" }

local RaidIcon = { 
	"yellow star",
	"orange circle",
	"purple diamond",
	"green triangle",
	"crescent moon",
	"blue square",
	"red X",
	"skull",
}

-- do we have all the units we need to do this? If no, return nil
-- if yes, return how many units that is
function ChatSubs.ValidUnits (emotestring)
  -- return 1 by default (the player)
  local valid = 1
  emotestring = string.lower(emotestring)

  if string.find(emotestring, "<target") then
    if UnitExists("target") then 
      valid = valid + 1
    else
      return nil
    end
  end
  if string.find(emotestring, "<focus") then
    if UnitExists("focus") then
      valid = valid + 1
    else 
      return nil
    end
  end
  if string.find(emotestring, "<pet") then
    if UnitExists("pet") then
      valid = valid + 1
    else
      return nil
    end
  end
  if string.find(emotestring, "<player1") then
    if UnitExists("party1") then valid = valid + 1 else return nil end
  end
  -- we don't have to check %p or <player -- there's always a player

  return valid
end

-- count the number of substitution units we have
function ChatSubs.CountSubs(string)
  local count = 0
  local match
  for match in string.gmatch(string, "<%w+>") do
    count = count + 1
  end
  return count
end

-- pass the text string we want <text> to use to ChatSubs
function ChatSubs.SetText(string)
  ChatSubs.CurrentText = string
end

-- UnitSex throws an error if we have no target, so let's wrap it...
local function Sex( who )
  if not UnitName(who) then
    return 1
  else
    return UnitSex(who)
  end
end

local function Capitalize( str )
  if str == "" or not str then return "" end
  return string.upper(str:sub(1, 1))..str:sub(2, -1)
end

-- expand our short target strings
function ChatSubs.whichtarget( str )
  local which
  which = str:sub(1, 1)
  if which == "t" or which == "target" then 
    return "target"
  elseif which == "f" then
    return "focus"
  elseif which == "m" then
    return "pet"
  elseif which == "o" then
    return "mouseover"
  else
    return "player"
  end
end

-- mobs don't have races, they have creature types
function ChatSubs.getrace( who )
  return UnitRace(who) or UnitCreatureType(who) or "unknown race"
end

-- for the <whowhat> syntax
function ChatSubs.substitute( str )
  local who, what
  local spacebefore = false

  if string.byte(str, 1) == 32 then
    spacebefore = true
    str = string.sub(str, 2)
  end

  -- temporary convert to lowercase, to ease our searching
  local testwho = string.lower(str)

  -- get the who part
  if string.find( testwho, "<player" ) then
    who = "player"
  elseif string.find( testwho, "<target" ) then
    who = "target"
  elseif string.find( testwho, "<pet" ) then
    who = "pet"
  elseif string.find( testwho, "<focus" ) then
    who = "focus"
  elseif string.find( testwho, "<mouseover" ) then
    who = "mouseover"
  elseif string.find( testwho, "<party%d" ) then
    local _, _, which = string.find( testwho, "<party(%d)" )
    who = "party"..which
  else
    -- didn't match any of those?  okay, let's see
    -- if we just have a "what"
    who = ""
  end
  -- now that we know the who part, the rest is the what part
  what = string.sub( str, string.len(who)+2, -2 )
  if who == "" then who = "player" end

-- uncomment for debugging
--  print(who.."-"..what)
  if not UnitExists(who) then
    who = "noone"
  end

  -- do we have a substitution for this?
  if ChatSubs[string.lower(what)] == nil then
    return str
  end

  local firstletter = string.sub( what, 1, 1 )
  local newstr
  if firstletter == string.upper(firstletter) then
    newstr = Capitalize( ChatSubs[string.lower(what)](who) )
  else
    newstr = ChatSubs[string.lower(what)](who)
  end

  if spacebefore then
    if newstr == "" then
      return ""
    else
      return " "..newstr
    end
  end

  return newstr
end

ChatSubs[""] = function (who)
  if who == "noone" then return "everyone" end
  return UnitName(who)
end

ChatSubs["name"] = ChatSubs[""]

ChatSubs["heshe"] = function (who)
  return Nominative[Sex(who)]
end

ChatSubs["himher"] = function (who)
  return Objective[Sex(who)]
end

ChatSubs["hisher"] = function (who)
  return Possessive[Sex(who)]
end

ChatSubs["hishers"] = function (who)
  return Possessive2[Sex(who)]
end

ChatSubs["sex"] = function (who)
  return Noun[Sex(who)]
end

ChatSubs["type"] = function (who)
  return string.lower((UnitCreatureFamily(who) or ChatSubs.getrace(who) or UnitCreatureType(who) or "thing"))
end

ChatSubs["text"] = function ()
  return ChatSubs.CurrentText
end

-- this works for English, but would have to be changed for localized versions
ChatSubs["class"] = function (who)
  local class
  _, class = UnitClass(who)
  if class == "DEATHKNIGHT" then 
    class = "death knight"
  end
  if class then 
    return string.lower(class) 
  else
    return "unknown class"
  end
end

ChatSubs["race"] = function (who)
  return string.lower(ChatSubs.getrace(who))
end

ChatSubs["level"] = function (who)
  return UnitLevel(who) or "unknown level"
end

ChatSubs["pvp"] = function (who)
  if UnitIsPVP(who) then
    return "flagged"
  else
    return "unflagged"
  end
end

ChatSubs["mark"] = function (who)
  return RaidIcon[GetRaidTargetIndex(who)] or "no icon"
end

ChatSubs["position"] = function(who)
  local x, y = GetPlayerMapPosition("player")
  return string.sub(x*100, 1, 4) .."," .. string.sub(y*100, 1, 4)
end

ChatSubs["health"] = function (who)
  return math.ceil(UnitHealth(who)/UnitHealthMax(who)*100) .. "%"
end

ChatSubs["boygirl"] = function(who)
  return ChildNoun[Sex(who)]
end

ChatSubs["ladlass"] = function(who)
  return LadNoun[Sex(who)]
end

ChatSubs["manwoman"] = function(who)
  return AdultNoun[Sex(who)]
end

ChatSubs["mastermistress"] = function(who)
  return MasterNoun[Sex(who)]
end

ChatSubs["lordlady"] = function(who)
  return LordNoun[Sex(who)]
end

ChatSubs["sirmadam"] = function(who)
  return SirNoun[Sex(who)]
end

-- declarations done, here's where we actually do the work.  It's 
-- pretty simple

-- save Blizzard's SendChatMessage so we can use it
local oldSendChatMessage = SendChatMessage

-- declare our own SendChatMessage
function SendChatMessage(msg, ...)
  -- search for substrings of the pattern <something>, subtitute for them
  msg = string.gsub(msg, "%s?<%w+>", ChatSubs.substitute)
  -- use blizzard's SendChatMessage to send the changed message
  oldSendChatMessage(msg, ...)
end
